@@ -57,7 +57,7 @@ def test_iterator():
         pass
     k = 0
     for (im,anno) in db.annotation.categorization:    
-        assert_equal(anno['id_img'], k)
+        assert_equal(anno['idx_image'], k)
         k += 1
     
 def test_split():
@@ -76,27 +76,27 @@ def test_split():
     # Show all annotations
     print '[test_viset.test_split]: show all annotations with step'              
     for (im, annotation) in db.annotation.categorization(step=2):
-        print 'Image=' + str(annotation['id_img']) + ', Category=' + annotation['category']
+        print 'Image=' + str(annotation['idx_image']) + ', Category=' + annotation['category']
         imshow(im)
     
     # Non iterated splits
     print '[test_viset.test_split]: random training/testing split'    
     (train, test) = db.annotation.categorization.split(step=1, strategy='train-test', randomize=True)
     for (im, annotation) in train:
-        print 'Training image=' + str(annotation['id_img']) + ', Category=' + annotation['category']        
+        print 'Training image=' + str(annotation['idx_image']) + ', Category=' + annotation['category']        
         imshow(im)
     for (im, annotation) in test:
-        print 'Testing image=' + str(annotation['id_img']) + ', Category=' + annotation['category']
+        print 'Testing image=' + str(annotation['idx_image']) + ', Category=' + annotation['category']
         imshow(im)
 
     # kfold splits
     for fold in db.annotation.categorization(step=1, strategy='kfold', folds=2):
         print '[test_viset.test_split]: kfold split - ' + str(len(fold))
         for (im, annotation) in fold.train:
-            print 'Training image=' + str(annotation['id_img']) + ', Category=' + annotation['category']
+            print 'Training image=' + str(annotation['idx_image']) + ', Category=' + annotation['category']
             imshow(im)
         for (im, annotation) in fold.test:
-            print 'Testing image=' + str(annotation['id_img']) + ', Category=' + annotation['category']
+            print 'Testing image=' + str(annotation['idx_image']) + ', Category=' + annotation['category']
             imshow(im)
 
     
