@@ -59,7 +59,7 @@ class Cache():
                 opt[key] = fragment[key]                
         return opt
   
-    def read(self, filename, reader=None, idx=None):
+    def _read(self, filename, reader=None, idx=None):
         """Read filename using supplied custom reader and item index and return object"""
         quietprint('[viset.cache.get][HIT]: Reading "' + reader + '" encoded object with index "' + str(idx) + '" from file "' + filename + '"', self._verbose)
         if reader == 'h5py':
@@ -74,7 +74,7 @@ class Cache():
             self.quietprint('[viset.cache.read][ERROR]: reader type "' + str(reader) + '" unsupported', self._verbose)
             raise NotImplementedError()
         
-    def unpack(self, filename, sha1=None, subpath=None):
+    def _unpack(self, filename, sha1=None, subpath=None):
         """Extract archive to archive directory, delete archive file and return archive directory"""
         if isarchive(filename):
             (unpackdir, ext) = viset.util.splitextension(filename)
@@ -88,14 +88,14 @@ class Cache():
                 filename = unpackdir    
         return filename
         
-    def load(self, filename, reader=None, idx=None):
+    def _load(self, filename, reader=None, idx=None):
         """Load from filename to an object"""
         if isimg(filename):
             # standard image formats
             obj = pylab.imread(filename)
         elif reader is not None:
             # custom reader defined in querystring
-            obj = self.read(filename, reader=reader, idx=idx)  
+            obj = self._read(filename, reader=reader, idx=idx)  
         elif path.isdir(viset.util.splitextension(filename)[0]):
             # archive directory in cache
             obj = viset.util.splitextension(filename)[0] 
@@ -111,13 +111,13 @@ class Cache():
             raise IOError()
         return obj
         
-    def download(self, url):        
+    def _download(self, url, timeout=None):        
         """Download url to cached file according to URL scheme and return filename in cache"""
-        self.free()  # garbage collection time?        
+        self._free()  # garbage collection time?        
         filename = self.cachefile(urlparse.urldefrag(url)[0])
         url_scheme = urlparse.urlparse(url)[0]
         if url_scheme == 'http':
-            viset.download.download(url, filename, verbose=self._verbose)                       
+            viset.download.download(url, filename, verbose=self._verbose, timeout=timeout)                       
         elif url_scheme == 'file':
             pass
         elif url_scheme == 'viset':
@@ -128,19 +128,19 @@ class Cache():
             raise NotImplementedError('FIXME: support for URL scheme ' + url_scheme)            
         return filename        
 
-    def fetch(self, url, sha1=None):
+    def _fetch(self, url, sha1=None, timeout=None):
         """Fetch a url and unpack in cache"""
         urlopts = self._url_fragment_options(url, sha1)
         if isurl(url):            
             if not self.iscached(url):
                 quietprint('[viset.cache][MISS]: downloading "' + str(url[0:63] + '...') + '"', self._verbose)                                        
-                filename = self.download(url)
+                filename = self._download(url, timeout)
                 if isarchive(filename):
                     quietprint('[viset.cache][MISS]: extracting archive \'' + filename + '\'', self._verbose)
                     if urlopts['subpath'] is not None:
-                        filename = self.unpack(filename, sha1=urlopts['sha1'], subpath=urlopts['subpath'])
+                        filename = self._unpack(filename, sha1=urlopts['sha1'], subpath=urlopts['subpath'])
                     else:
-                        filename = self.unpack(filename, sha1=urlopts['sha1'], subpath=None)                        
+                        filename = self._unpack(filename, sha1=urlopts['sha1'], subpath=None)                        
             else:
                 filename = self.cachefile(url)            
                 quietprint('[viset.cache.get][HIT]: fetching "' + url + '" from "' + filename + '"', self._verbose)                                                     
@@ -156,8 +156,33 @@ class Cache():
             return None
         
         quietprint('[viset.cache.get][HIT]: loading "' + filename + '"', self._verbose)                                         
-        return self.load(filename, reader=urlopts['reader'], idx=urlopts['idx'])
+        return self._load(filename, reader=urlopts['reader'], idx=urlopts['idx'])
 
+    def _hash(self, url, prettyhash=_prettyhash):
+        """Compute a SHA1 hash of a url to generate a unique cache filename"""
+        p = urlparse.urlsplit(url)
+        urlquery = urlparse.urlunsplit([p[0],p[1],p[2],p[3],None])
+        urlpath = urlparse.urlunsplit([p[0],p[1],p[2],None,None])        
+        (filename, ext) = viset.util.splitextension(urlpath)
+        urlopt = self._url_fragment_options(url)
+        urlhash = hashlib.sha1(urlquery).hexdigest()
+        if prettyhash:    
+            return path.basename(filename) + '_' + urlhash[0:7]
+        else:
+            return urlhash 
+
+    def _free(self):
+        """Garbage collection"""
+        if self._free_ctr == 0:
+            if self._cachesize is not None:
+                if self._cachesize.get() > self._maxsize:
+                    print 'WARNING: cachesize is larger than maximum.  Clean resources!'
+            quietprint('[viset.cache.free]: spawning cache garbage collection process', self._verbose)
+            self._cachesize = Pool(1).apply_async(self.size(), self._cacheroot)
+            self._free_ctr = self._free_maxctr
+        self._free_ctr -= 1
+
+        
     def put(self, obj, async=False, key='obj', filename=None):
         """Put a numpy object into cache and return url""" 
         if 'numpy' not in str(type(obj)):
@@ -179,30 +204,32 @@ class Cache():
             f.close()
         return CachedObject(url) if async else url
         
-    def get(self, url, sha1=None, async=False):
+    def get(self, url, sha1=None, async=False, timeout=None):
         """Get a url from cache with refetching on error and return object""" 
         if async:
             return CachedResult(url, sha1=sha1, verbose=self._verbose, cache=self)
         else:
             try:
-                return self.fetch(url, sha1=sha1)
+                return self._fetch(url, sha1=sha1)
             except KeyboardInterrupt:
                 pass
             except NotImplementedError:
                 raise
             except:
                 if self._refetch:
-                    quietprint('[viset.cache][ERROR]: cache fetch error - discarding and refetching', self._verbose)                                                     
+                    quietprint('[viset.cache][ERROR]: cache fetch exception - attempting to discard and refetch ... ', self._verbose)                                                     
                     self.discard(url)
-                    return self.fetch(url, sha1=sha1)
+                    return self._fetch(url, sha1=sha1)
                 else:
                     raise
             
     def discard(self, url):
         """Delete single url from cache"""
         if path.isfile(self.cachefile(url)):
+            quietprint('[viset.cache]: Removing cached file "' + self.cachefile(url) + '"', self._verbose)            
             os.remove(self.cachefile(url))
         if path.isdir(self.cachedir(url)):
+            quietprint('[viset.cache]: Removing cached directory "' + self.cachedir(url) + '"', self._verbose)                        
             shutil.rmtree(self.cachedir(url))
 
     def delete(self):
@@ -222,29 +249,6 @@ class Cache():
                 total_size += self.size(itempath)
         return total_size
 
-    def hash(self, url, prettyhash=_prettyhash):
-        """Compute a SHA1 hash of a url to generate a unique cache filename"""
-        p = urlparse.urlsplit(url)
-        urlquery = urlparse.urlunsplit([p[0],p[1],p[2],p[3],None])
-        urlpath = urlparse.urlunsplit([p[0],p[1],p[2],None,None])        
-        (filename, ext) = viset.util.splitextension(urlpath)
-        urlopt = self._url_fragment_options(url)
-        urlhash = hashlib.sha1(urlquery).hexdigest()
-        if prettyhash:    
-            return path.basename(filename) + '_' + urlhash[0:7]
-        else:
-            return urlhash 
-
-    def free(self):
-        """Garbage collection"""
-        if self._free_ctr == 0:
-            if self._cachesize is not None:
-                if self._cachesize.get() > self._maxsize:
-                    print 'WARNING: cachesize is larger than maximum.  Clean resources!'
-            quietprint('[viset.cache.free]: spawning cache garbage collection process', self._verbose)
-            self._cachesize = Pool(1).apply_async(self.size(), self._cacheroot)
-            self._free_ctr = self._free_maxctr
-        self._free_ctr -= 1
 
     def cachepath(self, url):
         """Return the filename of the url in the cache.  convenience function for for cachefile"""
@@ -256,7 +260,7 @@ class Cache():
         urlquery = urlparse.urlunsplit([p[0],p[1],p[2],p[3],None])        
         urlpath = urlparse.urlunsplit([p[0],p[1],p[2],None,None])
         urlopt = self._url_fragment_options(url)
-        urlhash = self.hash(url)
+        urlhash = self._hash(url)
         if urlopt['subpath'] is not None:
             return self.abspath(path.join(urlhash, urlopt['subpath']))
         else:
@@ -268,7 +272,7 @@ class Cache():
         p = urlparse.urlsplit(archive_url)
         urlpath = urlparse.urlunsplit([p[0],p[1],p[2],p[3],None])
         urlopt = self._url_fragment_options(archive_url)
-        urlhash = self.hash(urlpath)
+        urlhash = self._hash(urlpath)
         return self.abspath(urlhash)
         
     def abspath(self, basename):
@@ -302,12 +306,19 @@ class CachedObject(object):
         self._url = url
         self._sha1 = sha1        
     
-    def successful():
+    def successful(self):
         return (self._obj is not None)
-    def wait(timeout):
-        socket.setdefaulttimeout(timeout)  # do not set globally        
-    def ready():
+
+    def wait(self, timeout_in_seconds):
+        if self._obj is None:
+            self._obj = self._cache.get(self._url, sha1=self._sha1, async=False, timeout=timeout_in_seconds)
+        return self._obj
+    
+    def ready(self):
         return (self._obj is not None)
+
+    def url(self):
+        return self._url
     
     def get(self):
         if self._obj is None:
