@@ -41,6 +41,7 @@ class Viset(object):
             return self._cache
         for node in self._obj.root:
             if node._v_attrs.viewname == viewname:
+                # factory using class name in pytables node attribute
                 viewmodule = sys.modules['viset.stream']
                 viewclass = getattr(viewmodule, node._v_attrs.viewclass)
                 return viewclass(node, self)
@@ -52,9 +53,6 @@ class Viset(object):
     def __str__(self):
         return self._summary()
 
-    def flush(self):
-        self._obj.flush()
-
     def _summary(self):
         print 'Database URL: ' + self._dburl
         print 'Database name: ' + str(self.name())            
@@ -97,10 +95,15 @@ class Viset(object):
         else:
             raise IOError('Visets can only be opened in read "r",  read and append "r+" or write "w", mode, not "' + str(mode) + '"')                   
         
-    def close(self):
-        if self._obj is not None:
-            self._obj.flush()
-            self._obj.close()
+    def _isviset(self, obj):
+        try: 
+            if obj.root._v_attrs.viset == 'http://www.visym.com':
+                return True
+            else:
+                return False
+        except:
+            # does not contain required attribute
+            return False
 
     def _create(self, dbname, createviews=[ImageStream()]):
         """Create a new viset with the provided name and views"""
@@ -125,7 +128,16 @@ class Viset(object):
         # Return newly created database 
         obj.flush()
         return (obj, dbfile)
-        
+
+    def close(self):
+        if self._obj is not None:
+            self._obj.flush()
+            self._obj.close()
+            self._obj = None
+    
+    def flush(self):
+        self._obj.flush()
+    
     def name(self):
         return self._obj.root._v_attrs.viset_name        
 
@@ -136,15 +148,6 @@ class Viset(object):
     def version(self):
         return self._obj.root._v_attrs.viset_version
 
-    def _isviset(self, obj):
-        try: 
-            if obj.root._v_attrs.viset == 'http://www.visym.com':
-                return True
-            else:
-                return False
-        except:
-            # does not contain required attribute
-            return False
         
     def views(self):
         """List of views in database"""    
