@@ -9,7 +9,7 @@ def train_test(n, step=1, randomize=False, stratify=False):
         random.shuffle(k)
     k_train = list(k[:len(k)/2])
     k_test = list(k[len(k)/2:])
-    split = [(k_train, k_test, None)]                
+    split = {'train':k_train, 'test':k_test, 'validate':None}
     return split
 
 def kfold(n, folds, step=1, randomize=False, stratify=False):
@@ -27,7 +27,7 @@ def kfold(n, folds, step=1, randomize=False, stratify=False):
         k_test = list(k[u:v])
         k_train = list(k)  # copy required
         k_train[u:v] = []
-        split.append((k_train, k_test, None))
+        split.append({'train':k_train, 'test':k_test, 'validate':None})
     return split
 
 def leave_one_out(n, step=1, randomize=False, stratify=False):
@@ -40,4 +40,4 @@ def leave_zero_out(n, step=1, randomize=False, stratify=False):
     k = range(0, n, step)
     if randomize:
         random.shuffle(k)
-    return [(k, None, None)]                
+    return {'train':k, 'test':None, 'validate':None}
