@@ -25,6 +25,7 @@ class View(object):
     _db = None    
     _dbrow = None  # fast pytables writes
     _dbrowidx = None  # fast pytables writes    
+    _async = False
     
     def __init__(self, node=None, db=None):
         self._node = node
@@ -69,141 +70,112 @@ class GroupView(View):
             viewlist.append(str(node._v_pathname).replace('/','.'))
         return viewlist
     
-class KFoldStreamView(View):
-    """View for splitting a stream into kfold partitions"""
-    _split = None
-    _id_train = None
-    _id_test = None
-    _id_validate = None
-    _read = None
-    
-    def __init__(self, node=None, db=None, split=None, readfunc=None):
-        self._node = node
-        self._db = db
-        self._split = split
-        self._read = readfunc
-        
-    def __len__(self):
-        return len(self._split)
-
-    def __getattr__(self, name):
-        if name == 'train' and self._id_train is not None:
-            return SplitStreamView(self._node, self._db, self._id_train, readfunc=self._read)
-        elif name == 'test' and self._id_test is not None:
-            return SplitStreamView(self._node, self._db, self._id_test, readfunc=self._read)
-        elif name == 'validate' and self._id_validate is not None:
-            return SplitStreamView(self._node, self._db, self._id_validate, readfunc=self._read)            
-        else:  raise AttributeError, name
-
-    def __iter__(self):
-        return imap(self._set_fold, iter(range(len(self._split))))                 
-
-    def _set_fold(self, k):
-        """Internal function that sets a fold index without changing state"""
-        self._id_train = self._split[k][0]
-        self._id_test = self._split[k][1]
-        self._id_validate = self._split[k][2] 
-        return self
-
-    def __getitem__(self, k):
-        self._set_fold(k)
-        return self
-    
-class SplitStreamView(View):
-    """View for splitting a stream into training, testing and validation partitions"""
-    _id_split = None
-    _read = None
-    
-    def __init__(self, node=None, db=None, id_split=None, readfunc=None):
-        self._node = node
-        self._db = db
-        self._id_split = id_split
-        self._read = readfunc
-        
-    def __len__(self):
-        return len(self._id_split)
-
-    def __iter__(self):
-        return imap(self._read, iter(self._id_split))
-
-    def __getitem__(self, k):
-        return self._read(self._id_split[k])
-
-    
 class StreamView(View):
     """Stream view class"""
-    _split = None
-    _id = None
+    _fold = None
+    _split = {'train':None, 'test':None, 'validate':None, 'all':None}
+    _idx_stream = None
     
-    def __init__(self, node=None, db=None):
+    def __init__(self, node=None, db=None, idx_stream=None, split=None, async=False):
         self._node = node
         self._db = db
-        if self._node is not None:
-            self._set_split(strategy=None)
-            
+        self._idx_stream = idx_stream
+        self._async = async
+        self._split = split
+        
     def __len__(self):
-        if self._id is not None:
-            return len(self._id)
+        if self._idx_stream is not None:
+            return len(self._idx_stream)
         else:
             return self._node.nrows
 
+    def _get_viewclass(self):
+        viewmodule = sys.modules['viset.stream']
+        viewclass = getattr(viewmodule, self._node._v_attrs.viewclass)        
+        return viewclass
+    
     def __getattr__(self, name):
-        """Return dynamically constructed attributes"""        
-        if name == 'train' and self._split[0] is not None:
-            return SplitStreamView(self._node, self._db, id_split= self._split[0][0], readfunc=self.read)
-        elif name == 'test' and self._split[0] is not None:
-            return SplitStreamView(self._node, self._db, id_split= self._split[0][1], readfunc=self.read)
-        elif name == 'validate' and self._split[0] is not None:
-            return SplitStreamView(self._node, self._db, id_split= self._split[0][2], readfunc=self.read)
-        else:  raise AttributeError, name
-
+        """Attribute factory"""        
+        if self._split is not None:
+            viewclass = self._get_viewclass()
+            if name == 'train':
+                return viewclass(self._node, self._db, split=None, idx_stream=self._split['train'], async=self._async)
+            elif name == 'test':
+                return viewclass(self._node, self._db, split=None, idx_stream=self._split['test'], async=self._async)
+            elif name == 'validate':
+                return viewclass(self._node, self._db, split=None, idx_stream=self._split['validate'], async=self._async)
+            else:  raise AttributeError, name
+        else:  raise AttributeError, name                
+                
     def __iter__(self):
-        return imap(self.read, iter(self._id))
+        f = lambda x: self.read(x, async=self._async) 
+        if self._fold is not None:
+            return iter(self._fold)
+        elif self._idx_stream is not None:
+            return imap(f, iter(self._idx_stream))
+        else:
+            return imap(f, iter(range(len(self)))) # HACK                         
         
     def __getitem__(self, k):
-        return self.read(k)
+        f = lambda x: self.read(x, async=self._async) 
+        if self._fold is not None:
+            return self._fold[k]
+        elif self._idx_stream is not None:
+            return f(self._idx_stream[k])
+        else:
+            return f(k)
 
     def __call__(self, *args, **kwargs):
-        return self.split(*args, **kwargs)
+        self._set_split(*args, **kwargs)
+        return self        
     
-    def _set_split(self, strategy=None, split=None, folds=0, step=1, randomize=False, stratify=False):
+    def _set_split(self, strategy=None, folds=0, step=1, randomize=False, stratify=False, async=False):
         """Internal function that sets a split without returning copied split tuples"""
-        if split is not None:
-            self._split = split
-        elif strategy == 'kfold':            
+        self._async = async            
+        if strategy == 'kfold':            
             self._split = viset.partition.kfold(len(self), folds, step, randomize=randomize, stratify=stratify)
         elif strategy in ['leave-one-out', 'loo', 'LOO']:
             self._split = viset.partition.leave_one_out(len(self), step, randomize=randomize, stratify=stratify)
         elif strategy in ['train-test']:
             self._split = viset.partition.train_test(len(self), step, randomize=randomize, stratify=stratify)            
-        elif strategy in [None, 'all', 'leave_zero_out']:
-            split = viset.partition.leave_zero_out(len(self), step, randomize=randomize, stratify=stratify)            
-            self._id = split[0][0]
-            self._split = None 
-        else:
+        elif strategy in ['all', 'leave_zero_out'] or (step > 1):
+            self._split = viset.partition.leave_zero_out(len(self), step, randomize=randomize, stratify=stratify)            
+            self._idx_stream = self._split['train']
+            self._split = None
+        elif strategy is not None:
             IOError('unsupported splitting strategy ' + strategy)
-        self._split = viset.util.tolist(self._split)
-            
+        if self._split is not None:
+            viewclass = self._get_viewclass()
+            self._fold = [viewclass(self._node, self._db, split=split, idx_stream=None, async=self._async) for split in viset.util.tolist(self._split)]
+            if len(self._fold) > 1:
+                self._split = None
+
+
+    def where(self):
+        """FIXME: Support for returning iterators over rows matching complex query criteria using pytables 'where' syntax"""
+        # add a keyword 'where' to split which is used to create a pytables iterator
+        # this iterator is used instead of our iterators
+        pass
+    
     def split(self, *args, **kwargs):
         """Return a tuple containing split views"""
         self._set_split(*args, **kwargs)
-        if self._split[0] is None:
-            return self
-        elif len(self._split) > 1:
-            return copy.copy(KFoldStreamView(self._node, self._db, split=self._split, readfunc=self.read))
-        else:
-            if self._split[0][2] is not None:
-                return (copy.copy(SplitStreamView(self._node, self._db, id_split=self._split[0][0], readfunc=self.read)), 
-                        copy.copy(SplitStreamView(self._node, self._db, id_split=self._split[0][1], readfunc=self.read)), 
-                        copy.copy(SplitStreamView(self._node, self._db, id_split=self._split[0][2], readfunc=self.read)))
+        if self._split is not None:
+            if self._split['validate'] is not None:
+                return (copy.copy(self._fold[0].train), copy.copy(self._fold[0].test), copy.copy(self._fold[0].validate))
             else:
-                return (copy.copy(SplitStreamView(self._node, self._db, id_split=self._split[0][0], readfunc=self.read)), 
-                        copy.copy(SplitStreamView(self._node, self._db, id_split=self._split[0][1], readfunc=self.read)))
+                return (copy.copy(self._fold[0].train), copy.copy(self._fold[0].test))
+        elif self._fold is not None:
+            return self._fold
+        else:
+            return self
             
     def read(self):
         IOError('overloaded by subclass')
 
-
+    def flush(self):
+        self._node.flush()
+        
 class ImageStream(StreamView):
     class Table(tables.IsDescription):
         url = tables.StringCol(512, pos=0)   
@@ -228,10 +200,10 @@ class ImageStream(StreamView):
         self._dbrow.append()
         return self._dbrowidx
     
-    def read(self, idx):
+    def read(self, idx, async=False):
         imurl = self._node[idx]['url']	
         im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
-        if self._db._async:
+        if async or self._async:
             return im  
         else:
             return im.get()
@@ -261,12 +233,12 @@ class CategorizationAnnotationStream(StreamView):
         tbl.attrs.viewclass = 'CategorizationAnnotationStream' 
         dbobj.flush()
 
-    def read(self, idx):
+    def read(self, idx, async=False):
         idx_image = self._node[idx]['idx_image']
         imurl = self._db._obj.root.image[idx_image]['url']
         im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
         anno = self._node[idx]
-        if self._db._async:
+        if async or self._async:
             return (im, anno)  # numpy structured array tuple
         else:
             return (im.get(), anno)  # numpy structured array tuple
@@ -293,8 +265,8 @@ class CategorizationAnnotationStream(StreamView):
         pass
                     
     def labels(self):
-        """Unique category ids in dataset"""        
-        return np.unique(self._node.cols.idx_category[:])
+        """Unique category labels in dataset"""        
+        return list(set(self._node.cols.category[:]))
 
     def num_categories(self):
         return len(self.labels())
@@ -343,12 +315,12 @@ class DetectionAnnotationStream(StreamView):
         self._dbrow.append()
         return self._dbrowidx
 
-    def read(self, idx):
+    def read(self, idx, async=False):
         idx_image = self._node[idx]['idx_image']
         impath = self._db._obj.root.image[idx_image]['url']
         im = CachedImage(impath, cache=self._db._cache, verbose=self._db._verbose)
         anno = self._node[idx]
-        if self._db._async:
+        if async or self._async:
             return (im,anno)  # numpy structured array tuple
         else:
             return (im.get(),anno)  # numpy structured array tuple
@@ -394,7 +366,7 @@ class SegmentationAnnotationStream(StreamView):
         self._dbrow.append()
         return self._dbrowidx
         
-    def read(self, idx):
+    def read(self, idx, async=False):
         idx_image = self._node[idx]['idx_image']
         imurl = self._db._obj.root.image[idx_image]['url']	
         im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
@@ -404,7 +376,7 @@ class SegmentationAnnotationStream(StreamView):
         else:
 			mask = CachedImage(maskurl, cache=self._db._cache, verbose=self._db._verbose)
         anno = self._node[idx]
-        if self._db._async:
+        if async or self._async:
             anno = {'id_category':anno['idx_category'], 'category':anno['category'], 'mask':mask}            
             return (im,anno)  # numpy structured array tuple
         else:
@@ -435,12 +407,13 @@ class CategorizationOutputGroup(GroupView):
     
 class CategorizationOutputStream(StreamView):
     class Table(tables.IsDescription):
-        category = tables.StringCol(64, pos=0)              
-        idx_category = tables.Int32Col(pos=1)  
-        score = tables.Float32Col(pos=2)           
-        elapsed = tables.Float32Col(pos=3)   
-        idx_image = tables.Int64Col(pos=4)
-        imurl = tables.StringCol(512, pos=5)   
+        predlabel = tables.StringCol(64, pos=0)              
+        score = tables.Float32Col(pos=1)           
+        elapsed = tables.Float32Col(pos=2)   
+        imurl = tables.StringCol(512, pos=3)   
+        truelabel = tables.StringCol(64, pos=4)              
+        wn_predlabel = tables.StringCol(64, pos=5)           
+        wn_truelabel = tables.StringCol(64, pos=6)                                                         
 
     def _create(self, dbobj, viewname=None):
         if viewname is None:
@@ -449,31 +422,80 @@ class CategorizationOutputStream(StreamView):
         self._node.attrs.viewname = viewname
         self._node.attrs.viewclass = 'CategorizationOutputStream' 
         dbobj.flush()
+        # FIXME: rearchitect iterators from streamview to use where syntax, and use lazy evaluation to construct until iterator is requested so _set_split is not dependent on len
+        # len will be zero when we _create, so we cannot initialize _id for splits, need lazy evaluation to construct splits from options when we need them
         return self
 
-    def write(self, category, idx_category, score, elapsed, idx_image, imurl, imsubpath=None, imreader=None, imreaderindex=None):
+    def write(self, predlabel, score, elapsed, imurl, imsubpath=None, imreader=None, imreaderindex=None, truelabel=None, wn_predlabel=None, wn_truelabel=None):
         if self._dbrow is None:
             self._dbrow = self._node.row
             self._dbrowidx = 0
         else:
             self._dbrowidx += 1   
-        self._dbrow['category'] = category
-        self._dbrow['idx_category'] = idx_category
+        self._dbrow['predlabel'] = predlabel
         self._dbrow['score'] = score
-        self._dbrow['elapsed'] = elapsed        
+        self._dbrow['elapsed'] = elapsed
         self._dbrow['imurl'] = viset.util.join_fragment(imurl, viset.util.dict2querystring({'subpath':imsubpath, 'reader':imreader, 'index':imreaderindex}))
-        if idx_image is None:
-            self._dbrow['idx_image'] = self._dbrowidx
-        else:
-            self._dbrow['idx_image'] = idx_image        
+        self._dbrow['truelabel'] = truelabel
+        self._dbrow['wn_predlabel'] = wn_predlabel
+        self._dbrow['wn_truelabel'] = wn_truelabel                
         self._dbrow.append()
         return self._dbrowidx
 
-    def read(self, idx):
-        im = CachedImage(self._node[idx]['imurl'], cache=self._db._cache, verbose=self._db._verbose)
-        anno = self._node[idx]
-        if self._db._async:
-            return (im,anno)  # numpy structured array tuple
-        else:
-            return (im.get(),anno)  # numpy structured array tuple
     
+    def read(self, idx, async=False):
+        im = CachedImage(self._node[idx]['imurl'], cache=self._db._cache, verbose=self._db._verbose)
+        pred = {'label':self._node[idx]['predlabel'], 'score':self._node[idx]['score'], 'elapsed':self._node[idx]['elapsed']}
+        anno = {'label':self._node[idx]['truelabel']}
+        if async or self._async:
+            return (im,pred,anno)  # numpy structured array tuple
+        else:
+            return (im.get(),pred,anno)  # numpy structured array tuple
+
+
+    def num_categories(self):
+        return len(self.predlabels())
+        
+    def predlabels(self):
+        """Unique category labels in dataset"""        
+        return list(set(self._node.cols.predlabel[:]))
+
+    def truelabels(self):
+        """Unique category labels in dataset"""        
+        return list(set(self._node.cols.truelabel[:]))
+    
+
+    def y_pred(self, label, binary=False):
+        """Return a binary label vector y \in {-1,1} for dataset elements of a given category id"""        
+        predlabels = self._node.cols.predlabel[:]
+        if binary:
+            return np.int32(predlabels == label)  # {0,1}
+        else:
+            return 2*(np.int32(predlabels == label)) - 1  # {-1,1}
+
+    def y_true(self, label, binary=False):
+        """Return a binary label vector y \in {-1,1} for dataset elements of a given category id"""        
+        labels = self._node.cols.truelabel[:]
+        if binary:
+            return np.int32(labels == label)  # {0,1}
+        else:
+            return 2*(np.int32(labels == label)) - 1  # {-1,1}
+        
+    def Y_pred(self):
+        """Return a integer label vector Y \in {0,n} for predictions of a given category label index"""        
+        idx_predlabels = {k:v for (v,k) in enumerate(self.predlabels())}
+        labels = np.array(self._node.cols.predlabel[:])
+        Y = np.zeros(len(labels))
+        for (k, label) in enumerate(labels):
+            Y[k] = idx_predlabels[label]
+        return Y
+
+    def Y_true(self):
+        """Return a integer label vector Y \in {0,n} for truth of a given category label index"""        
+        idx_truelabels = {k:v for (v,k) in enumerate(self.truelabels())}
+        labels = np.array(self._node.cols.truelabel[:])
+        Y = np.zeros(len(labels))
+        for (k, label) in enumerate(labels):
+            Y[k] = idx_truelabels[label]
+        return Y
+
